#pragma once

#include "Shader.h"
#include "DX12Buffer.h"

struct ID3D12Device;
struct ID3D12GraphicsCommandList;
struct D3D12_INPUT_ELEMENT_DESC;
struct D3D12_ROOT_PARAMETER1;
struct D3D12_CONSTANT_BUFFER_VIEW_DESC;

namespace Framework
{
    class DX12Shader : public Shader
    {
    protected:
        ID3D12Device* device;
        DX12CopyCommandQueue* copy;
        DX12DirectCommandQueue* direct;
        unsigned char* shaderByteBuffer;
        int byteBufferSize;

    public:
        DX12Shader( ID3D12Device* device, DX12CopyCommandQueue* copy, DX12DirectCommandQueue* direct );
        virtual ~DX12Shader();
        //! erstellt ein constanten Buffer, der constante daten an den Shader bergibt
        //! es knnen maximal 14 Buffer erstellt werden
        //!  zD3d11Device: Das Device, mit dem der Buffer erstellt werden soll
        //! \param size Die gre des buffers in byte
        //! \param index Die position des Buffers im Buffer Array. Bereits vorhanderner Buffer wird ersetzt. Buffer 1 kann nicht erstellt werden, wenn Buffer 0 noch nicht erstellt wurde usw.
        virtual bool erstelleConstBuffer( int size, int index ) override;
        //! Setzt den Compilierten Shader
        //!  zD3d11Device: Das Device, mit welchem der Shader erstellt werden soll
        //! \param bytes Die Bytes des compilierten codes
        //! \param length die Lnge des bytearrays
        //! \return true, wenn bytes gltig ist, false sonst
        bool setCompiledByteArray( unsigned char* bytes, int length ) override;
        //! Nach dem Aufruf dieser Funktion wird dieser Shader als Pixel Shader benutzt
        //!  zD3d11Context: Das Context Objekt, mit dem der Shader verwendet werden soll
        void benutzeShader() override;
        //! gibt die compilierten bytes zurck
        unsigned char* getCompiledShader() const;
        //! gitbt die anzahl der compilierten bytes zurck
        int getCompiledLength() const;
        //! Erstellt den RootParameter zu einem constanten buffer
        //! \param index Der Index des Buffers
        //! \param view enthlt nach dem Aufruf die position und gre des buffers im speicher
        virtual void getViewDesc( int index, D3D12_CONSTANT_BUFFER_VIEW_DESC& view );
    };

    class DX12PixelShader : public DX12Shader
    {
    public:
        DX12PixelShader( ID3D12Device* device, DX12CopyCommandQueue* copy, DX12DirectCommandQueue* direct );
    };

    class DX12VertexShader : public DX12Shader
    {
    private:
        D3D12_INPUT_ELEMENT_DESC* inputLayout;
        int inputLayoutSize;

    public:
        //! Konstruktor
        DX12VertexShader( ID3D12Device* device, DX12CopyCommandQueue* copy, DX12DirectCommandQueue* direct );
        //! Destruktor
        ~DX12VertexShader();
        //! erstellt ein InputLayout fr den Shader
        //! Darf erst nach compile aufgerufen werden
        //!  zD3d11Device: Das Device, mit dem das Layout erstellt werden soll
        //! \param descArray Ein Array mit initialisierungsdaten
        //! \param anz Die Anzahl der Elemente im Array
        bool erstelleInputLayout( D3D12_INPUT_ELEMENT_DESC* descArray, int anz );
        //! Gint die Anzahl an eingabeparametern des Shaders zurck
        int getInputLayoutSize() const;
        //! Gibt eine Liste mit formaten fr jeden Eingabewert zurck
        D3D12_INPUT_ELEMENT_DESC* zInputLayout() const;
    };
}