#ifndef Datei_H
#define Datei_H

#include "Array.h"
#include "Reader.h"
#include "Writer.h"
#include <fstream>

namespace Framework
{
	class Text; //! Text.h
	class Zeit; //! Zeit.h
	namespace Encryption
	{
		class Key; //! Schlssel.h
	}
	class Datei; //! aus dieser Datei

	//! Liet und schreibt in eine Datei
	class Datei : public Reader, public Writer, public virtual ReferenceCounter
	{
	public:
		class Style
		{
		public:
			const static int lesen = 0x01; //! datei wird zum lesen geffnet
			const static int schreiben = 0x02; //! datei wirt zum schreiben geffnet
			const static int ende = 0x04; //! setzt dateizeiger ans Ende der Datei
		};
	private:
		std::fstream* stream;
		Text* pfad;
		__int64 gr;
		char tmpLByte;
		char tmpLBPos;
		char tmpSByte;
		char tmpSBPos;
		Encryption::Key* key;

	public:
		//! Konstruktor 
		DLLEXPORT Datei();
		//! Konstruktor 
		DLLEXPORT Datei(const char* pfad);
		//! Konstruktor 
		DLLEXPORT Datei(Text* pfad);
		//! Destruktor 
		DLLEXPORT virtual ~Datei();
		//! Setzt den Pfad zur Datei
		//! \param pfad Der Pfad
		DLLEXPORT void setDatei(const char* pfad);
		//! Setzt den Pfad zur Datei
		//! \param pfad Der Pfad
		DLLEXPORT void setDatei(Text* pfad);
		//! Benennt die Datei um oder verschiebt sie
		//! \param pfad Der neue Pfad zur Datei. Wenn sich nur der Name ndert, wird sie nur umbenannt
		//! \return 1, wenn das umbenennen erfolgreich war. 0, sonst
		DLLEXPORT bool umbenennen(const char* pfad);
		//! Benennt die Datei um oder verschiebt sie
		//! \param pfad Der neue Pfad zur Datei. Wenn sich nur der Name ndert, wird sie nur umbenannt
		//! \return 1, wenn das umbenennen erfolgreich war. 0, sonst
		DLLEXPORT bool umbenennen(Text* pfad);
		//! Lscht die Datei
		//! \return 1, wenn das lschen erfolgreich war. 0 sonst
		DLLEXPORT bool remove();
		//! Erstellt die Datei neu. Wenn im Pfad Ordner vorkommen, die nicht existieren, so werden sie erstellt
		//!  return 1: wenn das erstellen erfolgreich war. 0, sonst
		DLLEXPORT bool erstellen();
		//! ffnet die Datei
		//! \param style Legt fest, ob die Datei zum lesen und/oder schreiben geffnet wird. Alle Elemente aus Datei::Style:: sin mglich
		//!  return 1: wenn die datei erfolgreich geffnet werden konnte. 0 sonnst
		DLLEXPORT bool open(int style);
		//! Setzt die Position des Bytes, das als nchstes gelesen werden soll
		//! \param pos Der Index des Bytes
		//! \param ende 1, wenn der Index vom ende der Datei zhlt. 0, wenn der Index vom Beginn der Datei zhlt
		DLLEXPORT void setLPosition(__int64 pos, bool ende) override;
		//! Setzt die Position des Bytes, das als nchstes berschrieben wird
		//! \param pos Der Index des Bytes
		//! \param ende 1, wenn der Index vom ende der Datei zhlt. 0, wenn der Index vom Beginn der Datei zhlt
		DLLEXPORT void setSPosition(__int64 pos, bool ende) override;
		//! Schreibt in die Datei
		//! \param bytes Ein Array von bytes, die geschrieben werden sollen.
		//! \param len Wie viele Bytes in die Datei geschrieben werden sollen
		DLLEXPORT void schreibe(const char* bytes, int len) override;
		//! Liet aus der Datei
		//! \param bytes Ein Array, der mit Bytes aus der Datei gefllt werden soll
		//! \param len Wie viele Bytes aus der Datei gelesen werden sollen
		DLLEXPORT void lese(char* bytes, int len) override;
		//! Liet die nchste zeile der Datei ein
		//! \return Die gelesene Zeile als Text mit zeilenumbruch
		DLLEXPORT Text* leseZeile() override;
		//! Schliet die datei
		DLLEXPORT void close();
		//! Setzt den Schlssel fr die Datei
		DLLEXPORT void setKey(char* s, int l);
#ifdef WIN32
		//! Setzt den Zeitpunkt der letzten nderung der Datei (nur fr Windows)
		//! \param zeit den Zeitpunkt der letzten nderung
		//! \return 1, wenn der Zeitpunkt gesetzt wurde. 0 sonst
		DLLEXPORT bool setLetztenderung(Zeit* zeit);
#endif
		//! Liet das nchste Bit aus der Datei
		//! \param bit Eine Referenz auf deinen bool, in dem das Bit gespeichert wird
		//!  return 1, falls das lesen erfolgreich war. 0, sonst
		DLLEXPORT bool getNextBit(bool& bit);
		//! Speichert ein einzelnes Bit in der Datei
		//! \param bit Das bit, welches gespeichert werden soll
		//!  return 1, falls das speichern erfolgreich war
		DLLEXPORT bool setNextBit(bool bit);
		//! Prft, ob die Datei ein Verzeichnis ist
		//!  return 1, falls die Datei ein Verzeichnis ist. 0, sonst
		DLLEXPORT bool istOrdner() const;
		//! Prft, ob die Datei bereits geffnet wurde
		//! \return 1, wenn die Datei geffnet ist. 0 sonnst
		DLLEXPORT bool istOffen() const;
		//! Gibt die Anzahl an Unterdateien von dem Verzeichnis zurck
		//! \return 0, falls die Datei kein Verzeichnis ist. Sonst die Anzahl der Unterdateien
		DLLEXPORT int getUnterdateiAnzahl() const;
		//! Gibt eine Liste mit unterdateien zurck
		//!  return 0, falls die Datei kein Verzeichnis ist. Eine Liste mit den Namen der Unterdateien
		DLLEXPORT RCArray< Text >* getDateiListe() const;
		//! Gibt die Gre der Datei zurck
		//! return -1, falls die Datei ein Verzeichnis ist oder ein Fehler auftrat. Sonst die gre der Datei
		DLLEXPORT __int64 getSize() const;
		//! Gibt den Zeitpunkt der letzten nderung zurck
		//! \return 0, falls ein Fehler aufgetreten ist. Der Zeitpunkt der letzten nderung sonst
		DLLEXPORT Zeit* getLastChange() const;
		//! Prft, ob die Datei existiert
		//! \return 1, falls die Datei existiert. 0 sonnst
		DLLEXPORT bool existiert() const;
		//! Gibt den Index des Bytes aus der Datei zurck, welches als nchstes gelesen werden wrde
		//! return -1, falls ein Fehler aufgetreten ist. Sonst die Position des Lesezeigers
		DLLEXPORT __int64 getLPosition() const override;
		//! Gibt den Index des Bytes aus der Datei zurck, welches als nchstes berschrieben werden wrde
		//! return -1, falls ein Fehler aufgetreten ist. Sonst die Position des Schreibzeigers
		DLLEXPORT __int64 getSPosition() const override;
		//! Prft, ob die Datei vollstndig gelesen wurde
		//!  return 1, wenn die Datei vollstndig gelesen wurde. 0, sonst
		DLLEXPORT bool istEnde() const override;
		//! Gibt den Pfad zur Datei zurck
		DLLEXPORT Text* getPfad() const;
		//! Gibt den Pfad zur Datei ohne erhhten Reference Counter zurck
		DLLEXPORT Text* zPfad() const;
	};

	//! Datei Funktionen

	//! Sucht einen unbenutzten Dateinamen
	//! \param zPfad Ein Zeiger zu dem Pfad, in dem ein unbenutzter Dateiname gefunden werden soll. Ohne erhhten Reference Counter.
	//!          Wird auch als rckgabewert benutzt
	DLLEXPORT void GetFreePfad(Text* zPfad);
	//! Erstellt den vollstndigen Pfad mit Datei
	//! \param pfad Der Pfad zur Datei
	//! \return 1, falls das erstellen erfolgreich war.
	DLLEXPORT bool DateiPfadErstellen(Text* pfad);
	//! Lscht die angegebene Datei
	//! \param pfad Der Pfad zur Datei
	//! \return 1, falls die Datei gelscht wurde
	DLLEXPORT bool DateiRemove(Text* pfad);
	//! Benennt eine Datei um oder verschiebt sie
	//!  pfad_alt: Der Pfad zur Datei, die umbenannt werden soll.
	//!  pfad_neu: Der neue Pfad zur Datei. Wenn sich nur der Name ndert, wird sie nur umbenannt
	//! \return 1, wenn das umbenennen erfolgreich war. 0, sonst
	DLLEXPORT bool DateiUmbenennen(Text* pfad_alt, Text* pfad_neu);
	//! Prft, ob Datei existiert
	//! \param pfad Der Pfad zur Datei
	//! \return 1, wenn die Datei existiert. 0, wenn die Datei nicht gefunden wurde
	DLLEXPORT bool DateiExistiert(Text* pfad);
	//! prft, ob pfad ein Verzeichnis ist
	//! \param pfad Der Pfad zur Datei
	//! \return 1, wenn die Datei ein Verzeichnis ist. 0 sonst
	DLLEXPORT bool DateiIstVerzeichnis(Text* pfad);
	//! Erstellt den vollstndigen Pfad mit Datei
	//! \param pfad Der Pfad zur Datei
	//! \return 1, falls das erstellen erfolgreich war.
	DLLEXPORT bool DateiPfadErstellen(const char* pfad);
	//! Lscht die angegebene Datei
	//! \param pfad Der Pfad zur Datei
	//! \return 1, falls die Datei gelscht wurde
	DLLEXPORT bool DateiRemove(const char* pfad);
	//! Benennt eine Datei um oder verschiebt sie
	//!  pfad_alt: Der Pfad zur Datei, die umbenannt werden soll.
	//!  pfad_neu: Der neue Pfad zur Datei. Wenn sich nur der Name ndert, wird sie nur umbenannt
	//! \return 1, wenn das umbenennen erfolgreich war. 0, sonst
	DLLEXPORT bool DateiUmbenennen(const char* pfad_alt, const char* pfad_neu);
	//! Prft, ob Datei existiert
	//! \param pfad Der Pfad zur Datei
	//! \return 1, wenn die Datei existiert. 0, wenn die Datei nicht gefunden wurde
	DLLEXPORT bool DateiExistiert(const char* pfad);
	//! prft, ob pfad ein Verzeichnis ist
	//! \param pfad Der Pfad zur Datei
	//! \return 1, wenn die Datei ein Verzeichnis ist. 0 sonst
	DLLEXPORT bool DateiIstVerzeichnis(const char* pfad);
}

#endif