#ifndef Prozess_H
#define Prozess_H

#include "Betriebssystem.h"
#include "ReferenceCounter.h"

#ifdef WIN32
#pragma comment( lib, "Psapi.lib" )
#endif

namespace Framework
{
#ifdef WIN32
    struct MemoryInfo; //! aus dieser Datei
#endif
    class Prozess; //! aus dieser Datei

#ifdef WIN32
    struct MemoryInfo
    {
        unsigned long ausgelagerteFehler;
        __int64 hchsteAusgelagerterSpeicher;
        __int64 ausgelagerterSpeicher;
        __int64 hchsterAusgelagerterPool;
        __int64 ausgelagerterPool;
        __int64 hchsterNichtAusgelagerterPool;
        __int64 nichtAusgelagerterPool;
        __int64 vorreservierterSpeicher;
        __int64 hchsterVorreservierterSpeicher;
    };
#endif
    //! Diese Klasse findet informationen ber einen laufenden Prozess heraus (CPU, MEM)
    //! Bei Ubuntu immer der eigene Prozess
    class Prozess : public virtual ReferenceCounter
    {
    private:
#ifdef WIN32
        int numProcessors;
        ULARGE_INTEGER lastCPU, lastSysCPU, lastUserCPU;
        void *pHandle;
#endif

    public:
        //! Konstruktor 
        DLLEXPORT Prozess();
        //! nicht constant 
#ifdef WIN32
        //! Setzt den Prozess, der berwacht werden soll (Nur fr Windows)
        DLLEXPORT void setProcess( void *pHandle );
#endif
        //! Gibt den CPU verbrauch des Prozesses zurck
        DLLEXPORT double getCPU() const;
        //! Gibt den Speicherverbrauch des Prozesses zurck
        DLLEXPORT __int64 getMem() const;
#ifdef WIN32
        //! Gibt detailierte Informationen ber den Speicherverbrauch des Prozesses zurck (Nur fr Windows)
        DLLEXPORT MemoryInfo getMemInfo() const;
        //! Gibt die Anzahl der Threads zurck (Nur fr Windows)
        DLLEXPORT int getThreadAnzahl() const;
#endif
    };
}

#endif